/*--------------------------------------------------------------------------
 *  Copyright (c) 2006-2012 Igor Daniloff.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found in this directory.
 *------------------------------------------------------------------------*/

#ifndef DW_RS_SDK_ISERVER_H_
#define DW_RS_SDK_ISERVER_H_

#include "DwRsSDK_defs.H"
#include "DwRsSDK_IUnknown.H"

/**\file
   \~russian Поддержка общей функциональности сервера.
   \~english Support of the general server functionality. \~
*/

DWRSSDK_BEGIN

#ifdef __cplusplus

/**\~russian Базовый класс для поддержки сервера.
   Работает в многопоточной среде.
   \~english Base class for the server support.
   Operates in multi-thread environment.
*/
class IServer:virtual public IUnknown
{
public:

    virtual ~IServer(){}

    /**\~russian Инициализация настройки сервера. Должна вызываться
    перед любой функцией данного класса и функциями его производных
    классов. Допускается повторный вызов (к примеру, при перечитывании
    конфигурации), но при этом все потоки, вызывающие IServer::Process,
    должны быть заблокированы через IServer::Block.
    После успешной инициализации потоки следует снова разблокировать
    через IServer::Unblock.
    \~english Initialization of the server settings. Must be called before any function
    of the given class and functions of its derived classes. Repeated call is allowed
    (for example, to re-read the configuration), but all the threads calling for IServer::Process
    must be blocked via IServer::Block. After successful initialization these threads
    should be unblocked via IServer::Unblock. \~

    \param[in] base_dir - \~russian строка с адресом базовой директории MailD (параметр
    BaseDir из секции General конфигурационного файла Dr.Web MailD). Должна
    заканчиваться знаком '/'. При установке по умолчанию значение должно быть
    равно '/var/drweb/'.
    \~english a string with the path to the base directory of MailD (the BaseDir parameter from General section
    of Dr.Web MailD configuration file). Must end with the '/' symbol. If Dr.Web MailD was
    installed with default settings, the value of this parameter is equal to '/var/drweb/'. \~

    \param[in] id - \~russian уникальный идентификатор компонента. Директория с
    таким именем будет создаваться в /var/drweb/msgs/out (по умолчанию) с
    точкой в начале. Также будут созданы сокеты в /var/drweb/ipc (по умолчанию), в
    состав имени которых будет входить данный идентификатор.
    \~english unique identifier of the component. It will be used in the name of the directory
    created in /var/drweb/msgs/out by default (with '.' symbol before it). Also it will be used
    as a part of socket names (these sockets will be created in /var/drweb/ipc by default).
    \~

    \param[in] timeout - \~russian значение таймаута (в секундах) на
    работу с одним клиентом.
    \~english value of the timeout (in seconds) for operation with one client. \~

    \param[in] stat - \~russian собирать ли статистику.
     \~english  whether to collect statistics. \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
    */
    virtual bool Initialize(const char* base_dir, const char* id, int timeout, bool stat )=0;

    /**\~russian Сохраняет статистику по использованию соединений в
       заданный файл.
    \~english Collects statistics on the use of connections to the specified file.\~

       \param[in] file - \~russian имя файла, в который должна быть сохранена статистика.
    \~english name of the file to which statistics is saved.\~

       \return \~russian 1 - при успехе, 0 - при неудаче.
    \~english 1 - at success, 0 - at failure.  \~
     */
    virtual bool DumpStat(const char* file)=0;

    /**\~russian Функция блокирует все потоки работающие в IServer::Process
     до вызова IServer::Unblock.
       \~english The function blocks all threads working in IServer::Process
     until IServer::Unblock is called. \~

       \param[in] timeout - \~russian таймаут (в секундах) на выполнение
       блокировки.
       \~english timeout (in seconds) for blocking. \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
    */
    virtual bool Block(int timeout)=0;
    /**\~russian Разблокирует выполнение всех потоков, заблокированных
       перед этим через IServer::Block.
       \~english Unblocks execution of all threads blocked earlier via IServer::Block. \~
    */
    virtual void Unblock(void)=0;
    /**\~russian Посылает запрос на остановку потока, работающего в IServer::Process.
      Данный поток выходит из IServer::Process с кодом ::PR_StopThread. Эта функция
      не ждет завершения работы потока - такую проверку должно выполнить само приложение.
       \~english Sends a query to stop a thread working in IServer::Process.
       This thread exits the IServer::Process with the PR_StopThread code. This function does not wait for the
       thread to terminate its operation - the application must check this itself. \~

       \param[in] tid - \~russian идентификатор потока для остановки.
       \~english identifier of a thread for termination. \~
    */
    virtual void StopThread(const void* tid)=0;

    /**\~russian Обработка запросов клиента. Должна вызываться в бесконечном цикле.
       При завершении с кодом ::PR_StopThread приложение должно выйти из цикла и
       завершить выполнение потока.
       \~english Processing of client queries. Must be called in an infinite loop. If ended with
       the ::PR_StopThread code, the application must exit the loop and terminate execution
       of the thread. \~

       \param[in] timeout - \~russian таймаут, в течение которого нужно ожидать запросов
       от клиентов. 0 - не устанавливать таймаут. Если произошел таймаут, то будет возвращено
       ::PR_StopThread.
       \~english timeout for awaiting requests from clients. 0 - timeout is not set. If timeout is expired, then
       ::PR_StopThread is returned. \~

       \return \~russian Код завершения обработки.
       \~english Code of processing termination. \~
     */
    virtual ProcessRes Process(int timeout=0)=0;

};

#endif

DWRSSDK_END

#endif
