/*--------------------------------------------------------------------------
 *  Copyright (c) 2006-2012 Igor Daniloff.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found in this directory.
 *------------------------------------------------------------------------*/

#ifndef DW_RS_SDK_PTR_H_
#define DW_RS_SDK_PTR_H_

#include "DwRsSDK_defs.H"

/**\file
   \~russian Поддержка автоматического освобождения памяти в C++.
   \~english Support of automatic memory deallocation in C++.
*/

DWRSSDK_BEGIN

#ifdef __cplusplus

/**\~russian Умный указатель для объектов со включенными счетчиками ссылок.
    \~english Smart pointer for objects with enabled reference counters.
 */
template <class T>
class DwPtr
{
public:
    /**\~russian Инициализация пустым значением.
       \~english Initialization by an empty value. \~
     */
    DwPtr():m_ptr (0)
    {}

    /**\~russian Инициализация заданным указателем.
       \~english Initialization by a predetermined pointer. \~
       \param[in] p - \~russian Значение указателя.
       \~english Pointer value. \~
       \param[in] add_ref - \~russian true, если после инициализации нужно увеличить значение
       счетчика ссылок, и false в противном случае.
       \~english true, if the reference counter value should be increased after the initialization, otherwise it is false. \~
     */
    DwPtr(T* p, bool add_ref=true)
    {
        if ((m_ptr = p) != 0 && add_ref)
            m_ptr->AddRef();
    }
    /**\~russian Инициализация другим значением DwPtr. Потоковая безопасность
       должна обеспечиваться отдельно!
       \~english Initialization by another DwPtr value. Thread security must be provided separately! \~

       \param[in] p - \~russian объект DwPtr, от которого производится инициализация.
       \~english a DwPtr object , from which initialization is made. \~
     */
    DwPtr(const DwPtr& p)
    {
        if ((m_ptr = p.m_ptr) != 0)
            m_ptr->AddRef();
    }

    ~DwPtr() /*throw ()*/
    {
        if (m_ptr)
            m_ptr->Release();
    }

    /**\~russian Возвращает значение указателя.
    \~english Returns the value of the pointer. \~
       \return \~russian Значение указателя, 0 - если указатель не был инициализирован.
       \~english The pointer value, 0 - if the pointer was not initialized. \~

     */
    T * get() const
    {
        return m_ptr;
    }

    /**\~russian Возвращает ссылку на объект. Последний должен быть инициализирован!
    \~english Returns the reference to the object. The latter must be initialized! \~
     */
    T& operator*() const /*throw ()*/
    {
        return *m_ptr;
    }
    /**\ref DwPtr::get
     */
    T* operator->() const /*throw ()*/
    {
        return m_ptr;
    }

    /**\~russian Устанавливает новое значение указателя. Старое значение
       при необходимости удаляется.
       \~english Sets the new value of the pointer. The old value is deleted, if necessary. \~

       \param[in] p - \~russian новое значение указателя.
       \~english new value of the pointer. \~
       \return \~russian Новое значение указателя.
       \~english New value of the pointer. \~
     */
    T* operator=(T* p) /*throw ()*/
    {
        return set_(&m_ptr, p);
    }

    /**\~russian Устанавливает новое значение указателя. Старое значение
      при необходимости удаляется.
       \~english Sets the new value of the pointer. The old value is deleted, if necessary. \~


       \param[in] p - \~russian новое значение указателя.
       \~english new value of the pointer. \~
       \return \~russian Новое значение указателя.
       \~english New value of the pointer. \~
     */
    template<class Y>
    T* operator=(const DwPtr<Y>& p) /*throw ()*/
    {
        return set_(&m_ptr, p.m_ptr);
    }
    /**\~russian Устанавливает новое значение указателя. Старое значение
      при необходимости удаляется.
       \~english Sets the new value of the pointer. The old value is deleted, if necessary.\~


       \param[in] p - \~russian новое значение указателя.
       \~english new value of the pointer.\~
       \return \~russian Новое значение указателя.
       \~english New value of the pointer.\~
     */
    T* operator=(const DwPtr& p) /*throw ()*/
    {
        return set_(&m_ptr, p.m_ptr);
    }

    /**\~russian  Проверяет инициализирован ли объект.
    \~english  Checks if the object is initialized. \~
       \return \~russian true, если объект не инициализирован, и false в противном случае.
       \~english true, if the object was not initialized, otherwise it is false. \~
     */
    bool operator!() const /*throw ()*/
    {
        return (m_ptr == 0);
    }
private:
    template<class Y>
    T* set_(T** pp, Y* p) /*throw ()*/
    {
        if (p != 0)
            p->AddRef();
        if (*pp)
            (*pp)->Release();
        *pp = p;
        return p;
    }

    T*    m_ptr;
};

#endif

DWRSSDK_END

#endif
